/*
 * $Id: noiz2sa.c,v 1.8 2003/02/12 13:55:13 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Noiz2sa main routine.
 *
 * @version $Revision: 1.8 $
 */
//#include "SDL.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <psppower.h>
#include <pspiofilemgr_fcntl.h>
#include <pspiofilemgr.h>
#include "png.h"
#include "psplib.h"
#include "noiz2sa.h"
#include "screen.h"
//#include "vector.h"
#include "ship.h"
#include "shot.h"
#include "frag.h"
#include "bonus.h"
//#include "foe_mtd.h"
#include "foe.h"
//#include "brgmng_mtd.h"
#include "barragemanager.h"
#include "background.h"
#include "degutil.h"
#include "soundmanager.h"
#include "attractmanager.h"
#include "colors.h"

static int noSound = 0;

PSP_MODULE_INFO("NOIZ2SA", 0, 1, 1);
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER);

#define printf	pspDebugScreenPrintf
#define BUF_WIDTH (512)
#define SCR_WIDTH (480)
#define SCR_HEIGHT (272)

u16 *VRAM=(u16*)(void *)(0x44000000); 

int volume=64;


int exit_callback(int arg1, int arg2, void *common)
{
	quitLast();
	return 0;
}

int CallbackThread(SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}

int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

// Initialize and load preference.
static void initFirst() {
  loadPreference();
  //srand(SDL_GetTicks());
  srand(1);
  setColors();
  initBarragemanager();
  initAttractManager();
}

// Quit and save preference.
void quitLast() {
 // if ( !noSound ) {
//	stopMusic();
//	closeSound();
 // }
  scePowerSetClockFrequency(222,222,111);
  printf("saving scores\n");
  savePreference();
  /*printf("closing xml patterns\n");
  closeBarragemanager();
  printf("closing graphics\n");
  freeSprites(); 
*/
  sceKernelExitGame();
}

int status;

static float stagePrm[STAGE_NUM+ENDLESS_STAGE_NUM+1][3] = {
  {13, 0.5f, 0.12f}, {2, 1.8f, 0.15f}, {3, 3.2f, 0.1f}, {90, 6.0f, 0.3f}, {5, 5.0f, 0.6f},
  {6, 10.0f, 0.6f}, {7, 5.0f, 2.2f}, {98, 12.0f, 1.5f}, {9, 10.0f, 2.0f}, {79, 21.0f, 1.5f},
  {-3, 5.0f, 0.7f}, {-1, 10.0f, 1.2f}, {-4, 15.0f, 1.8f}, {-2, 16.0f, 1.8f},
  {0, -1.0f, 0.0f},
};

void initTitleStage(int stg) {
  initFoes();
  initBarrages(stagePrm[stg][0], stagePrm[stg][1], stagePrm[stg][2]);
}

void initTitle() {
  int stg;
  status = TITLE;
  stg = initTitleAtr();
  initShip();
  initShots();
  initFrags();
  initBonuses();
  initBackground();
  setStageBackground(1);
  initTitleStage(stg);
}

void initGame(int stg) {
  status = IN_GAME;

  initShip();
  initShots();
  initFoes();
  initFrags();
  initBonuses();
  initBackground();

  initBarrages(stagePrm[stg][0], stagePrm[stg][1], stagePrm[stg][2]);
  initGameState(stg);
  if ( stg < STAGE_NUM ) {
    setStageBackground(stg%5+1);
    playMusic(stg+1);
  } else {
    if ( !insane ) {
      setStageBackground(0);
      playMusic(0);
    } else {
      setStageBackground(6);
      playMusic(11);
    }
  }
}

void initGameover() {
  status = GAMEOVER;
  initGameoverAtr();
}

void initStageClear() {
  status = STAGE_CLEAR;
  initStageClearAtr();
}

static void move() {
  switch ( status ) {
  case TITLE:
    moveTitleMenu();
    moveBackground();
    addBullets();
    moveFoes();
    break;
  case IN_GAME:
    moveBackground();
    addBullets();
    moveShots();
    moveShip();
    moveFoes();
    moveFrags();
    moveBonuses();
    break;
  case GAMEOVER:
    moveGameover();
    moveBackground();
    addBullets();
    moveShots();
    moveFoes();
    moveFrags();
    break;
  case STAGE_CLEAR:
    moveStageClear();
    moveBackground();
    moveShots();
    moveShip();
    moveFrags();
    moveBonuses();
    break;
  case PAUSE:
    movePause();
    break;
  }
}

static void draw() {
  switch ( status ) {
  case TITLE:
    // Draw background.
    drawBackground();
    drawFoes();
    drawBulletsWake();
    blendScreen();
    drawBullets();
    drawScore();
    //drawRPanel();
    drawTitleMenu();
    break;
  case IN_GAME:
    // Draw background.
    drawBackground();
    drawBonuses();
    drawFoes();
    drawBulletsWake();
    drawFrags();
    blendScreen();
    // Draw forground.
    drawShots();
    drawShip();
    drawBullets();
    drawScore();
    //drawRPanel();
    break;
  case GAMEOVER:
    // Draw background.
    drawBackground();
    drawFoes();
    drawBulletsWake();
    drawFrags();
    blendScreen();
    // Draw forground.
    drawShots();
    drawBullets();
    drawScore();
    //drawRPanel();
    drawGameover();
    break;
  case STAGE_CLEAR:
    // Draw background.
    drawBackground();
    drawBonuses();
    drawFrags();
    blendScreen();
    // Draw forground.
    drawShots();
    drawShip();
    drawScore();
    //drawRPanel();
    drawStageClear();
    break;
  case PAUSE:
    // Draw background.
    drawBackground();
    drawBonuses();
    drawFoes();
    drawBulletsWake();
    drawFrags();
    blendScreen();
    // Draw forground.
    drawShots();
    drawShip();
    drawBullets();
    drawScore();
    //drawRPanel();
    drawPause();
    break;
  }
}


static int accframe = 0;

/*
static void usage(char *argv0) {
  fprintf(stderr, "Usage: %s [-nosound] [-window] [-reverse] [-brightness n] [-nowait] [-accframe]\n", argv0);
}

static void parseArgs(int argc, char *argv[]) {
  int i;
  for ( i=1 ; i<argc ; i++ ) {
    if ( strcmp(argv[i], "-nosound") == 0 ) {
      noSound = 1;
    } else if ( strcmp(argv[i], "-window") == 0 ) {
      windowMode = 1;
    } else if ( strcmp(argv[i], "-reverse") == 0 ) {
      buttonReversed = 1;
    } else if ( (strcmp(argv[i], "-brightness") == 0) && argv[i+1] ) {
      i++;
      brightness = (int)atoi(argv[i]);
      if ( brightness < 0 || brightness > 256 ) {
	brightness = DEFAULT_BRIGHTNESS;
      }
    } else if ( strcmp(argv[i], "-nowait") == 0 ) {
      nowait = 1;
    } else if ( strcmp(argv[i], "-accframe") == 0 ) {
      accframe = 1;
    } else {
      usage(argv[0]);
      exit(1);
    }
  }
}
*/
int tick=0;
int interval = INTERVAL_BASE;
static int pPrsd = 1;

int main(void) {
   float curr_ms = 1.0f;
   float curr_fps;
   int val=0;
   int timer;
  struct timeval time_slices[16];
  int done = 0;
  long prvTickCount = 0;
  int i;
  int btn;
  //SDL_Event event;
  long nowTick;
  int frame;
  char image_path[80];
  int screen_counter =0;

  	SetupCallbacks();
        scePowerSetClockFrequency(333,333,166);
	pspDebugScreenInit();
	print_logo();
	printf("Initializing...");
  initDegutil();
  initSDL();
  if ( !noSound ) initSound();
  initFirst();
  initTitle();

  
  SceCtrlData pad;
  sceCtrlSetSamplingCycle(0);
  sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

  clearAllScreen();
  clearPanels();
  guInit();

  while ( !done ) {
    //SDL_PollEvent(&event);
    sceCtrlPeekBufferPositive(&pad, 1); 
    //keys = SDL_GetKeyState(NULL);
    if (pad.Buttons &  PSP_CTRL_LTRIGGER) {
	sprintf(image_path,"ms0:/noiz2sa_screen%d.png",screen_counter++);
	screenshot(image_path);
    }
    if (pad.Buttons &  PSP_CTRL_SQUARE){
	if (volume>0)
		volume--;
	setChunkVolume(volume);
    } else if  (pad.Buttons &  PSP_CTRL_TRIANGLE){
	if (volume<128)
		volume++;
	setChunkVolume(volume);
    }
    if (pad.Buttons & PSP_CTRL_START){
      if ( !pPrsd ) {
	if ( status == IN_GAME ) {
	  status = PAUSE;
	  pauseMusic();
	} else if ( status == PAUSE ) {
	  status = IN_GAME;
	  resumeMusic();
	}
      }
      pPrsd = 1;
    } else {
      pPrsd = 0;
    }
/*
   curr_fps = 1.0f / curr_ms;
     pspDebugScreenSetXY(0,0);
     pspDebugScreenPrintf("%d.%03d",(int)curr_fps,(int)((curr_fps-(int)curr_fps) * 1000.0f));
    if (curr_fps<15)
	frame++;
    else if (curr_fps>20)
	frame--;

   if (frame>5)
	frame=5;
   if (frame < 1) {
	frame=1;
	for (timer=0; timer<3000; timer++)
		timer++;
		// do nothing
   } 

   gettimeofday(&time_slices[val & 15],0);

                val++;

                if (!(val & 15))
                {
                        struct timeval last_time = time_slices[0];
                        unsigned int i;

                        curr_ms = 0;
                        for (i = 1; i < 16; ++i)
                        {
                                struct timeval curr_time = time_slices[i];

                                if (last_time.tv_usec > curr_time.tv_usec)
                                {
                                        curr_time.tv_sec++;
                                        curr_time.tv_usec-=1000000;
                                }

                                curr_ms += ((curr_time.tv_usec-last_time.tv_usec) + (curr_time.tv_sec-last_time.tv_sec) * 1000000) * (1.0f/1000000.0f);

                                last_time = time_slices[i];
                        }
                        curr_ms /= 15.0f;
                }
*/

   //frame=3;
  // for ( i=0 ; i<frame ; i++ ) {
      move();
      tick++;
	if (RENDER_BG) { 
      		move();
      		tick++;
	}
 //  }
    //smokeScreen();
    clearScreen();
    draw();
    flipScreen();
  }
  quitLast();
  return 0;
}


void screenshot(const char* filename) 
{ 
   png_structp png_ptr; 
   png_infop info_ptr; 
   FILE* fp; 
   int i, x, y; 
   u16* data = VRAM; 
   int width=SCR_WIDTH; 
   int height=SCR_HEIGHT; 
   int linesize=BUF_WIDTH; 
   int saveAlpha=0; 
   u8* line; 

   if ((fp = fopen(filename, "wb")) == NULL) return; 
   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL); 
   if (!png_ptr) return; 
   info_ptr = png_create_info_struct(png_ptr); 
   if (!info_ptr) { 
      png_destroy_write_struct(&png_ptr, (png_infopp)NULL); 
      return; 
   } 
   png_init_io(png_ptr, fp); 
   png_set_IHDR(png_ptr, info_ptr, width, height, 8, 
      saveAlpha ? PNG_COLOR_TYPE_RGBA : PNG_COLOR_TYPE_RGB, 
      PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT); 
   png_write_info(png_ptr, info_ptr); 
   line = (u8*) malloc(width * (saveAlpha ? 4 : 3)); 
   for (y = 0; y < height; y++) { 
      for (i = 0, x = 0; x < width; x++) { 
         u16 color = data[x + y * linesize]; 
         int r = (color & 0x1f) << 3; 
         int g = ((color >> 5) & 0x1f) << 3 ; 
         int b = ((color >> 10) & 0x1f) << 3 ; 
         line[i++] = r; 
         line[i++] = g; 
         line[i++] = b; 
         if (saveAlpha) { 
            int a = color & 0x8000 ? 0xff : 0; 
            line[i++] = a; 
         } 
      } 
      png_write_row(png_ptr, line); 
   } 
   free(line); 
   png_write_end(png_ptr, info_ptr); 
   png_destroy_write_struct(&png_ptr, (png_infopp)NULL); 
   fclose(fp); 
} 

void print_logo() {
	printf("\n \
  _   _  ___ ___ _________  ____    _      ____  ____  ____   \n \ 
 | \\ | |/ _ \\_ _|__  /___ \\/ ___|  / \\    |  _ \\/ ___||  _ \\  \n \ 
 |  \\| | | | | |  / /  __) \\___ \\ / _ \\   | |_) \\___ \\| |_) | \n \
 | |\\  | |_| | | / /_ / __/ ___) / ___ \\  |  __/ ___) |  __/  \n \ 
 |_| \\_|\\___/___/____|_____|____/_/   \\_\\ |_|   |____/|_|     \n \ 
\n\n \
				version 0.6 \n\n \
			created by Kenta Cho (ABA GAMES)\n \
  			  PSP port by Denis Televnyy\n\n\n\n\n\n\n\n \
"); 

}
